@@ -1,6 +1,4 @@
 #ifndef DISCRETECM_H_
 #define DISCRETECM_H_
-#include "distribution.h"
-
-
+#include "cdf.h"
 #endif
